// URLCheckerDoc.cpp : implementation of the CURLCheckerDoc class
//

#include "stdafx.h"
#include "URLChecker.h"

#include "URLCheck.h"
#include "UrlObj.h"
#include "URLCheckerDoc.h"
#include "URLCheckerView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CURLCheckerDoc

IMPLEMENT_DYNCREATE(CURLCheckerDoc, CDocument)

BEGIN_MESSAGE_MAP(CURLCheckerDoc, CDocument)
	//{{AFX_MSG_MAP(CURLCheckerDoc)
	ON_COMMAND(ID_FILE_EXPORT, OnFileExport)
	ON_COMMAND(ID_FILE_IMPORT, OnFileImport)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CURLCheckerDoc construction/destruction

CURLCheckerDoc::CURLCheckerDoc() : m_nSuspendIndex(-1), m_nOldObjState(CUrlObj::UNDEFINED)
{
}

CURLCheckerDoc::~CURLCheckerDoc()
{
}

BOOL CURLCheckerDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;
	m_nSuspendIndex = (-1);
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CURLCheckerDoc serialization

void CURLCheckerDoc::Serialize(CArchive& ar)
{
	CUrlObj* pUrlObj = NULL;
	int nSize = 0;

	if (ar.IsStoring())
	{
		nSize = m_arrUrls.GetSize();
		ar << m_nSuspendIndex;
		ar << m_nOldObjState;
		ar << nSize;
		if (nSize)
		{
			for (int nItem = 0; nItem < nSize; nItem++)
			{
				pUrlObj = (CUrlObj*)m_arrUrls.GetAt(nItem);
				if (pUrlObj != NULL)
					pUrlObj->Serialize(ar); 
			}
			pUrlObj = NULL;
		}
	}
	else
	{
		RemoveAllObjs();
		
		HWND hWndList = NULL;
		CURLCheckerView* pView = GetView();
		if (pView != NULL)
			hWndList = pView->m_hWndList;

		ar >> m_nSuspendIndex;
		ar >> m_nOldObjState;
		ar >> nSize;
		if (nSize)
		{
			for (int nItem = 0; nItem < nSize; nItem++)
			{
				pUrlObj = new CUrlObj;
				if (pUrlObj != NULL)
				{
					pUrlObj->Serialize(ar);
					pUrlObj->AddToListView(hWndList);
					m_arrUrls.Add((CObject*)pUrlObj); 
				}
			}
			pUrlObj = NULL;
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
// CURLCheckerDoc diagnostics

#ifdef _DEBUG
void CURLCheckerDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CURLCheckerDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CURLCheckerDoc commands

CURLCheckerView* CURLCheckerDoc::GetView()
{
	CURLCheckerView* pView = NULL;
	POSITION pos = GetFirstViewPosition();
	if (pos != NULL)
	{
		pView = (CURLCheckerView*)GetNextView(pos);
	}
	return pView;
}

void CURLCheckerDoc::DeleteContents() 
{
	RemoveAllObjs();
	CDocument::DeleteContents();
}

void CURLCheckerDoc::RemoveAllObjs()
{
	// Stop check process
	CURLCheckerApp* App = (CURLCheckerApp*)AfxGetApp();
	VERIFY(App != NULL);
	App->StopProgress(); 

	// DeleteAllItems in ListView
	CURLCheckerView* pView = GetView();
	if (pView != NULL)
		pView->DeleteAllItems();
	
	// m_arrUrls.RemoveAll()
	int nBound = m_arrUrls.GetUpperBound();
	if (nBound != -1)
	{
		// m_arrUrls.RemoveAll
		CUrlObj* pUrlObj = NULL;
		for (int nItem = nBound; nItem > -1; nItem--)
		{
			pUrlObj = (CUrlObj*)m_arrUrls.GetAt(nItem);
			if (pUrlObj != NULL)
				delete pUrlObj;
		}
		pUrlObj = NULL;
		m_arrUrls.RemoveAll(); 
	}
	m_nSuspendIndex = (-1);
}

void CURLCheckerDoc::UpdateState(int nItem, int nType)
{
	if (nItem < m_arrUrls.GetSize())
	{
		CUrlObj* pUrlObj = (CUrlObj*)m_arrUrls.GetAt(nItem);
		if (pUrlObj != NULL)
			UpdateAllViews(NULL, (LPARAM)nType, (CObject*)pUrlObj); 
	}
}

BOOL CURLCheckerDoc::SetAborted(int nIndex, CUrlObj* pObj)
{
	VERIFY(nIndex != (-1));
	VERIFY(pObj != NULL);
	m_nSuspendIndex = nIndex;
	m_nOldObjState = pObj->m_nState; 
	pObj->m_nState = CUrlObj::ABORTED;
	return TRUE;
}

BOOL CURLCheckerDoc::ClearAborted()
{
	if (m_nSuspendIndex == (-1))
		return TRUE;
	CUrlObj* pObj = (CUrlObj*)m_arrUrls.GetAt(m_nSuspendIndex);
	VERIFY(pObj != NULL);
	pObj->m_nState = m_nOldObjState;
	m_nOldObjState = CUrlObj::UNDEFINED;
	m_nSuspendIndex = (-1);
	return TRUE;
}

void CURLCheckerDoc::OnFileImport() 
{
	// Check Modified
	if (IsModified())
	{
		if(IDYES == AfxMessageBox(_T("Wish previously to save the changed datas?"), MB_YESNO))
		{
			if (!SaveModified())
				return ;
		}
		SetModifiedFlag(FALSE);
	}

	// Open file
	LPCTSTR lpszFilter = _T("Text Files (*.txt)|*.txt||");
	CFileDialog fileDlg(TRUE, NULL, NULL,
		OFN_HIDEREADONLY|OFN_OVERWRITEPROMPT, lpszFilter);
	if (!fileDlg.DoModal()) return ;
	
	CString strFileName = fileDlg.GetPathName();
	if (strFileName.GetLength() == 0) return ;

	FILE* stream = _tfopen(LPCTSTR(strFileName), "rt");
	if (stream == NULL)
	{
		return ;
	}
	
	// Delete current contents
	RemoveAllObjs();
	
	// Get hWndList  
	HWND hWndList = NULL;
	CURLCheckerView* pView = GetView();
	if (pView != NULL)
		hWndList = pView->m_hWndList;

	// Read from text-file
	const int nSize = 3*_MAX_PATH;
	_TCHAR szbuffer[nSize];
	LPCTSTR lpCurPoz = NULL;
	LPCTSTR lpNextPoz = NULL;
	_TCHAR szCode[10];
	_TCHAR szURL[_MAX_PATH];
	_TCHAR szDescription[_MAX_PATH];
	int nLen = 0;
	int nSep = 0;
	BOOL bIsCode = FALSE;
	BOOL bIsUrl = FALSE;
	while (_fgetts(szbuffer, nSize, stream) != NULL)
	{
		bIsCode = FALSE;
		bIsUrl = FALSE;
		lpCurPoz = szbuffer;
		*szCode = NULL;
		lpNextPoz = _tcschr(lpCurPoz, '\t');
		if (lpNextPoz != NULL)
		{
			nLen = int(lpNextPoz-lpCurPoz);
			if (*lpCurPoz == '"')
			{
				nSep = 1;
				nLen -= 2;
			}
			else
				nSep = 0;
			_tcsncpy(szCode, lpCurPoz+nSep, (size_t)nLen);
			*(szCode+nLen) = NULL;
			lpCurPoz = lpNextPoz+1;
			bIsCode = (nLen > 0);
		}
		*szURL = NULL;
		lpNextPoz = _tcschr(lpCurPoz, '\t');
		if (lpNextPoz != NULL)
		{
			nLen = int(lpNextPoz-lpCurPoz);
			if (*lpCurPoz == '"')
			{
				nSep = 1;
				nLen -= 2;
			}
			else
				nSep = 0;
			_tcsncpy(szURL, lpCurPoz+nSep, (size_t)nLen);
			*(szURL+nLen) = NULL;
			lpCurPoz = lpNextPoz+1;
			bIsUrl = bIsCode && (nLen > 0);
		}
		*szDescription = NULL;
		lpNextPoz = _tcschr(lpCurPoz, '\t');
		if (lpNextPoz == NULL)
		{
			// It's realy last column
			lpNextPoz = szbuffer+_tcslen(szbuffer)-1;
		}
		if (lpNextPoz != NULL)
		{
			nLen = int(lpNextPoz-lpCurPoz);
			if (*lpCurPoz == '"')
			{
				nSep = 1;
				nLen -= 2;
			}
			else
				nSep = 0;
			_tcsncpy(szDescription, lpCurPoz+nSep, (size_t)nLen);
			*(szDescription+nLen) = NULL;
			lpCurPoz = lpNextPoz+1;
		}

		if (bIsUrl)
		{
			// Add new object
			m_arrUrls.Add((CObject*)new CUrlObj(atoi(szCode), szURL, szDescription, hWndList)); 
		}
	}
	fflush(stream);
	fclose(stream);
}

void CURLCheckerDoc::OnFileExport() 
{
	// Open file
	LPCTSTR lpszFilter = _T("Text Files (*.txt)|*.txt||");
	CFileDialog fileDlg(TRUE, NULL, NULL,
		OFN_HIDEREADONLY|OFN_OVERWRITEPROMPT, lpszFilter);
	if (!fileDlg.DoModal()) return ;
	
	CString strFileName = fileDlg.GetPathName();
	if (strFileName.GetLength() == 0) return ;

	FILE* stream = _tfopen(LPCTSTR(strFileName), "wt");
	if (stream == NULL)
	{
		return ;
	}

	int nSize = m_arrUrls.GetSize(); 
	if (nSize)
	{
		_TCHAR szCode[10];

		CUrlObj* pUrlObj = NULL;
		for (int nItem = 0; nItem < nSize; nItem++)
		{
			pUrlObj = (CUrlObj*)m_arrUrls.GetAt(nItem);
			if (pUrlObj != NULL)
			{
				// Code
				if(_stprintf(szCode, _T("%d"), pUrlObj->m_nCode) == EOF)
					break;
				if(_fputts(szCode, stream) == EOF)
					break;
				if(_fputts(_T("\t\""), stream) == EOF)
					break;
				
				// Url
				if(_fputts(LPCTSTR(pUrlObj->m_strURL), stream) == EOF)
					break;
				if(_fputts(_T("\"\t\""), stream) == EOF)
					break;
				
				// Description
				if(_fputts(LPCTSTR(pUrlObj->m_strDes), stream) == EOF)
					break;
				if(_fputts(_T("\"\t"), stream) == EOF)
					break;
				
				// State
				if(_fputts(pUrlObj->GetState(pUrlObj->m_nState), stream) == EOF)
					break;
				if(_fputts(_T("\t"), stream) == EOF)
					break;
				
				// StatusCode
				if(_stprintf(szCode, _T("%d\n"), pUrlObj->m_dwStatusCode) == EOF)
					break;
				if(_fputts(szCode, stream) == EOF)
					break;
			}
		}
		pUrlObj = NULL;
	}
	fclose(stream);
}
