// URLCheckerView.cpp : implementation of the CURLCheckerView class
//

#include "stdafx.h"
#include "URLChecker.h"

#include "URLCheck.h"

#include "UrlObj.h"
#include "URLCheckerDoc.h"
#include "URLCheckerView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

COLUMNINFO CURLCheckerView::m_columns[] =
{
	_T("Code"), 40,
	_T("URL"), 245,
	_T("Description"), 200,
	_T("State"), 100
};

/////////////////////////////////////////////////////////////////////////////
// CURLCheckerView

IMPLEMENT_DYNCREATE(CURLCheckerView, CListView)

BEGIN_MESSAGE_MAP(CURLCheckerView, CListView)
	//{{AFX_MSG_MAP(CURLCheckerView)
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CListView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CListView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CListView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CURLCheckerView construction/destruction

CURLCheckerView::CURLCheckerView() : m_pListCtrl(NULL), m_hWndList(NULL)
{
}

CURLCheckerView::~CURLCheckerView()
{
}

BOOL CURLCheckerView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CListView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CURLCheckerView drawing

void CURLCheckerView::OnDraw(CDC* pDC)
{
	CURLCheckerDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here
}

void CURLCheckerView::OnInitialUpdate()
{
	CListView::OnInitialUpdate();
	m_pListCtrl = (CListCtrl*)&GetListCtrl();
	VERIFY(m_pListCtrl != NULL);
	m_hWndList = GetSafeHwnd();
	VERIFY(m_hWndList != NULL);
	
	DWORD dwStyle = ListView_GetExtendedListViewStyle(m_hWndList);
	dwStyle |= LVS_EX_GRIDLINES|LVS_EX_FULLROWSELECT;
	ListView_SetExtendedListViewStyle(m_hWndList, dwStyle);
	
	AddColumns();
}

/////////////////////////////////////////////////////////////////////////////
// CURLCheckerView printing

BOOL CURLCheckerView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CURLCheckerView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CURLCheckerView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CURLCheckerView diagnostics

#ifdef _DEBUG
void CURLCheckerView::AssertValid() const
{
	CListView::AssertValid();
}

void CURLCheckerView::Dump(CDumpContext& dc) const
{
	CListView::Dump(dc);
}

CURLCheckerDoc* CURLCheckerView::GetDocument() // non-debug version is inline
{
	VERIFY(m_pDocument->IsKindOf(RUNTIME_CLASS(CURLCheckerDoc)));
	return (CURLCheckerDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CURLCheckerView message handlers

BOOL CURLCheckerView::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext) 
{
	dwStyle |= LVS_REPORT|LVS_SHOWSELALWAYS;
	return CWnd::Create(lpszClassName, lpszWindowName, dwStyle, rect, pParentWnd, nID, pContext);
}

void CURLCheckerView::DeleteAllItems()
{
	if (m_hWndList != NULL)
		if (ListView_GetItemCount(m_hWndList))
			ListView_DeleteAllItems(m_hWndList);
}

BOOL CURLCheckerView::IsColumns()
{
	if (m_hWndList == NULL)
		return FALSE;
	
	HWND hwndHD = ListView_GetHeader(m_hWndList);
	if (hwndHD != NULL)
	{
		if(Header_GetItemCount(hwndHD) == 4)
			return TRUE;
	}
	return FALSE;
}

BOOL CURLCheckerView::AddColumns()
{
	VERIFY(m_hWndList != NULL);
	if (IsColumns())
		return TRUE;
	
	LVCOLUMN col;
	memset((void*)&col, 0, sizeof(LVCOLUMN));
	col.mask = LVCF_TEXT|LVCF_WIDTH|LVCF_FMT;
	col.fmt = LVCFMT_LEFT;
	
	for (int nColumn = 0; nColumn < sizeof(m_columns)/sizeof(COLUMNINFO); nColumn++)
	{
		col.pszText = (LPTSTR)m_columns[nColumn].lpszCaption;   
		col.cx = m_columns[nColumn].nWidth;   
		VERIFY(ListView_InsertColumn(m_hWndList, nColumn, &col) != -1); 
	}
	return TRUE;
}

void CURLCheckerView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	CUrlObj* pUrlObj = (CUrlObj*)pHint;
	if (pUrlObj == NULL)
		return ;
	if (lHint == 2)
		pUrlObj->UpdateState(m_hWndList, (int)lHint, FALSE); 
	else
	{
		CURLCheckerApp* pApp = (CURLCheckerApp*)AfxGetApp();
		VERIFY(pApp != NULL);
		pUrlObj->UpdateState(m_hWndList, (int)lHint, pApp->m_bAutoScroll); 
	}
}

void CURLCheckerView::OnSize(UINT nType, int cx, int cy) 
{
	CListView::OnSize(nType, cx, cy);
	
	CURLCheckerApp* pApp = (CURLCheckerApp*)AfxGetApp();
	VERIFY(pApp != NULL);
	if (!pApp->m_bAutoSizeColumns)
		return ;

	if (!IsColumns())
		return ;

	int nRest = cx;
	LVCOLUMN col;
	memset((void*)&col, 0, sizeof(LVCOLUMN));
	col.mask = LVCF_WIDTH; 
	VERIFY(ListView_GetColumn(m_hWndList, 0, &col));
	nRest -=col.cx;
	VERIFY(ListView_GetColumn(m_hWndList, 3, &col));
	nRest -=col.cx;
	if (nRest > 100)
	{
		double dcxColUrl = 0;
		double dcxColDes = 0;
		VERIFY(ListView_GetColumn(m_hWndList, 1, &col));
		dcxColUrl = (double)col.cx;
		VERIFY(ListView_GetColumn(m_hWndList, 2, &col));
		dcxColDes = (double)col.cx;
		int ncxUrlDes = int(dcxColUrl+dcxColDes);
		dcxColUrl /= ncxUrlDes;
		dcxColDes /= ncxUrlDes;

		col.cx = int(nRest*dcxColUrl);
		VERIFY(ListView_SetColumn(m_hWndList, 1, &col));
		col.cx = int(nRest*dcxColDes);
		VERIFY(ListView_SetColumn(m_hWndList, 2, &col));
	}
}